@@ -48,7 +48,7 @@
 #include <ifaddrs.h>
 #ifdef __linux__
 	#include <netpacket/packet.h>
-#elif defined (__FreeBSD__) || defined(__NetBSD__) || defined (__OpenBSD__) || defined(__APPLE__)
+#elif defined (__FreeBSD__) || defined(__NetBSD__) || defined (__OpenBSD__) || defined(__APPLE__) || ( !defined(__FreeBSD__) && defined(__FreeBSD_kernel__))
 	#include <net/if_dl.h>
 #endif
 #include <sys/select.h>
